import numpy
import constant


def advection(plon, plat, lon, lat, u, v, tadv, dt):
    # Initialize trajectory lists
    traj_time = [0]
    traj_lon = [plon]
    traj_lat = [plat]
    time = 0

    # Convert dt in seconds
    dt_s = dt * constant.daytosec

    #Loop on Advection time
    while time < tadv:
        time += dt
        dist = (((lon - plon)*numpy.cos(numpy.deg2rad(lat)))**2
                + (lat - plat)**2)
        dist_min = numpy.min(dist)
        u_time = u[round(time)]
        v_time = v[round(time)]

        # Convert velocity in deg/s:
        u_time, v_time = converttoangular(lon, lat, numpy.array(u_time), numpy.array(v_time))
        u_time[:, 0] = 0
        u_time[:, -1] = 0
        v_time[0, :] = 0
        v_time[-1, :] = 0
        #u_time = u_time / constant.degtom #(constant.degtom * numpy.cos(numpy.deg2rad(lat)))
        #v_time = v_time / constant.degtom

        i, j = numpy.where(dist == dist_min)
        plon = plon + u_time[i, j] * dt_s
        plat = plat + v_time[i, j] * dt_s
        traj_lon.append(plon)
        traj_lat.append(plat)
        traj_time.append(time)
    return traj_lon, traj_lat, traj_time


def tconcatenate(listu, listv, time):
    ut = []
    vt = []
    for t in range(time):
        ut.append(u)
        vt.append(v)
    return ut, vt


def advect_ensemble_point(grid_x, lon, lat, u, v):
    xt=[]
    yt=[]
    for x, y in enumerate(grid_x[0], grid_x[1]):
        xtmp, ytmp = advection(x, y, lon, lat, u, v)
        xt.append(x)
        yt.append(y)
    return xt, yt

def converttoangular(x, y, u, v):
    import constant
    """
    Convert U V components from metric to angular system units                  
    """                                                                         
    assert(u.shape == v.shape)                                                  
                                                                                
    nx = len(x)                                                                 
    ny = len(y)                                                                 
                                                                                
    # Keep longitude between -180, 180                                          
    x[x > 180] -= 360                                                           
    x0 = x                                                                      
    y0 = y                                                                      
                                                                          
    # Conversion from spherical to cartesian coordinates and move it            
    # for one second using U and V component                                    
    lon = numpy.radians(x0)                                                     
    lat = numpy.radians(y0)                                                     
    sin_x = numpy.sin(lon)                                                      
    cos_x = numpy.cos(lon)                                                      
    sin_y = numpy.sin(lat)                                                      
    cos_y = numpy.cos(lat)                                                      
    xc = -u * sin_x - v * cos_x * sin_y                                         
    yc = u * cos_x - v * sin_y * sin_x                                          
    zc = v * cos_y                                                              
    xc += constant.Rearth * cos_y * cos_x                                          
    yc += constant.Rearth * cos_y * sin_x                                          
    zc += constant.Rearth * sin_y                                                  
                                                                                
    # Conversion from cartesian to spherical coordinates                        
    x1 = numpy.degrees(numpy.arctan2(yc, xc))                                   
    y1 = numpy.degrees(numpy.arcsin(                                            
                       zc / numpy.sqrt(xc * xc + yc * yc + zc * zc)))           
    dx = x1 - x0                                                                
    dy = y1 - y0                                                                
                                                                                
    # Return the velocity in degrees/s                                          
    return (dx + 180) % 360 - 180, dy                                           
                                       